package queue;

/**
 * Dynamická struktura fronta umožňující vkládání a vyjímání prvků.
 */
public class Queue<E> {

	/**
	 * První prvek ve frontě.
	 */
	private Item<E> head;

	/**
	 * Poslední prvek ve frontě.
	 */
	private Item<E> tail;

	/**
	 * Jeden prvek fronty.
	 */
	private static class Item<E> {

		/**
		 * Data uložená v tomto prvku fronty.
		 */
		private E data;

		/**
		 * Reference na následující prvek ve frontě.
		 */
		private Item<E> next;

		/**
		 * Vytvoří novou instanci prvku fronty.
		 * 
		 * @param data
		 *            data uložená v novém prvku.
		 */
		public Item(E data) {
			this.data = data;
			next = null;
		}

		/**
		 * Vrátí data uložená v tomto prvku fronty.
		 * 
		 * @return data z tohoto prvku.
		 */
		public E getData() {
			return data;
		}

		/**
		 * Vrátí následující prvek ve frontě.
		 * 
		 * @return následující prvek.
		 */
		public Item<E> getNext() {
			return next;
		}

		/**
		 * Nastaví následující prvek ve frontě.
		 * 
		 * @param next
		 *            následující prvek ve frontě.
		 */
		public void setNext(Item<E> next) {
			this.next = next;
		}
	}

	/**
	 * Vrací true, pokud je fronta prázdná, false jinak.
	 * 
	 * @return true, pokud je fronta prázdná, false jinak.
	 */
	public boolean isEmpty() {
		return head == null;
	}

	/**
	 * Vloží prvek na konec fronty.
	 * 
	 * @param data
	 *            obsah vkládaného prvku.
	 */
	public void enqueue(E data) {
		Item<E> insert = new Item<E>(data);
		if (tail != null) {
			tail.setNext(insert);
			tail = insert;
		} else {
			head = insert;
			tail = insert;
		}
	}

	/**
	 * Vyjme první prvek ze začátku fronty.
	 * 
	 * @return data vyjímaného prvku.
	 * @throws IllegalStateException
	 *             pokud je fronta prázdná.
	 */
	public E dequeue() {
		if (head == null) {
			throw new IllegalStateException(
					"Nelze vybrat prvek z prázdné fronty.");
		}
		E retval = head.getData();
		head = head.getNext();
		if (head == null) {
			tail = null;
		}
		return retval;
	}

}
